#if false
//#define LosFormatter

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Drawing;
using System.Drawing.Design;

using System.Reflection;
using System.Threading;
using System.Web;
using System.Globalization;
using System.IO;

using Westwind.Tools;

namespace Westwind.Web.Controls
{
    /// <summary>
    /// Control used to automatically preserve properties on a page.
    /// By calling PreserveProperty you're instructing this control
    /// to persist and then restore the value of the specified property
    /// or field of a control and have it automatically restored on the
    /// next page access.
    /// 
    /// The difference of this mechanism is that it works with ViewState
    /// off, and automatically reassigns persisted values back to the 
    /// controls they belong to without any extra code. It greatly 
    /// simplifies persisting values like IDs or some ViewState based
    /// values without having to use Viewstate on a page.
    /// </summary>
    [ParseChildren(true)]
    [PersistChildren(false)]
    public class PreservePropertyControl2 : Control
    {

        /// <summary>
        /// Collection of all the preserved properties that are to
        /// be preserved/restored. Collection hold, ControlId, Property
        /// </summary>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        public List<PreservedProperty> PreservedProperties
        {
            get
            {
                return _PreservedProperties;
            }
        }
        List<PreservedProperty> _PreservedProperties = new List<PreservedProperty>();



        /// <summary>
        /// Required to be able to properly PreservedProperty Collection 
        /// </summary>
        /// <param name="obj"></param>
        protected override void AddParsedSubObject(object obj)
        {
            if (obj is PreservedProperty)
                this.PreservedProperties.Add(obj as PreservedProperty);
        }

        /// <summary>
        /// Determines whether the control preserves and restores values
        /// </summary>
        [Description("Determines whether the control preserves and restores values"),DefaultValue(true)]
        public bool Enabled
        {
            get
            {
                return _Enabled;
            }
            set
            {
                _Enabled = value;
            }
        }
        private bool _Enabled = true;

        /// <summary>
        /// When true stores page state in a session variable.
        /// </summary>
        [Description("When true stores page state in a session variable"),DefaultValue(false)]
        public bool StoreInSession
        {
            get
            {
                return _StoreInSession;
            }
            set
            {
                _StoreInSession = value;
            }
        }
        private bool _StoreInSession = false;

        /// <summary>
        /// Determines whether the entire buffer is base64 Encoded.
        /// Defaults to true, but you can turn this off for debugging
        /// to see the content more easily.
        /// </summary>
        public bool Base64Encode
        {
            get
            {
                return _Base64Encode;
            }
            set
            {
                _Base64Encode = value;
            }
        }
        bool _Base64Encode = true;

        /// <summary>
        /// Adds a control to the collection. At this point only the
        /// control and property are stored.
        /// </summary>
        /// <param name="WebControl"></param>
        /// <param name="Property"></param>
        /// <returns></returns>
        public bool PreserveProperty(Control WebControl, string Property)
        {
            PreservedProperty pp = new PreservedProperty();
            pp.ControlId = WebControl.UniqueID;
            pp.ControlInstance = WebControl;
            pp.Property = Property;

            this.PreservedProperties.Add(pp);
            return true;
        }

        /// <summary>
        /// Adds a control to the collection. At this point only the
        /// control and property are stored.
        /// </summary>
        /// <param name="ControlId"></param>
        /// <param name="Property"></param>
        /// <returns></returns>
        public bool PreserveProperty(string ControlId, string Property)
        {
            Control ctl = this.Page.FindControl("ControlId");
            if (ctl == null)
                return false;

            return this.PreserveProperty(ctl, Property);
        }

        /// <summary>
        /// Read in data of preserved properties in OnInit
        /// </summary>
        /// <param name="e"></param>
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            if (this.Enabled)
                this.RestoreOutput();
        }

        /// <summary>
        /// Write out data for preserved properties in OnPreRender
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);
            if (this.Enabled)
                this.RenderOutput();
        }

        protected override void Render(HtmlTextWriter writer)
        {
            if (this.DesignMode)
                writer.Write("[ *** wwPersister: " + this.ID + " *** ]");

            base.Render(writer);
        }

        /// <summary>
        /// Restores the Preserved Properties back to their
        /// respective controls if possible.
        /// </summary>
        protected void RestoreOutput()
        {
            if (!this.Page.IsPostBack)
                return;

            /*
            `|btnSubmit~|ForeColor~|Green|``|btnSubmit~|Font~|Tahoma, 12pt|``|txtName~|Style~|System.Web.UI.CssStyleCollection|``|__Page~|Test~|Hello|``|__Page~|Value~|2|`
            */
            string RawBuffer = null;

            if (this.StoreInSession)
            {
                RawBuffer = HttpContext.Current.Session["__PreserveProperty"] as string;
                if (string.IsNullOrEmpty(RawBuffer))
                    return;
            }
            else
            {
                // *** Hidden Form Variable
                RawBuffer = HttpContext.Current.Request.Form["__" + this.UniqueID];
                if (string.IsNullOrEmpty(RawBuffer))
                    return;
            }

            // *** Note if we're storing in the Session object 
            // *** no Base64Encoding of the final string occurs no matter what the flag says
            if (this.Base64Encode && !this.StoreInSession)
                RawBuffer = Encoding.UTF8.GetString(Convert.FromBase64String(RawBuffer));

            string[] PropertyStrings = RawBuffer.Split(new string[1] { "`|" }, StringSplitOptions.RemoveEmptyEntries);
            foreach (string TPropertyString in PropertyStrings)
            {
                // *** Strip off |`
                string PropertyString = TPropertyString.Substring(0, TPropertyString.Length - 2);

                string[] Tokens = PropertyString.Split(new string[1] { "~|" }, StringSplitOptions.None);
                string ControlId = Tokens[0];
                string Property = Tokens[1];
                string StringValue = Tokens[2];

                Control Ctl = this.Page.FindControl(ControlId);
                if (Ctl == null)
                    continue;

#if LosFormatter
                object Value = this.LosDeserializeObject(StringValue);
#else
                // *** Get existing Value
                object CtlValue = wwUtils.GetPropertyCom(Ctl, Property);

                object Value = this.StringToTypedValue(StringValue, CtlValue.GetType(), CultureInfo.InvariantCulture);
                if (Value == null)
                    continue;
#endif

                wwUtils.SetPropertyCom(Ctl, Property, Value);
            }
        }


        /// <summary>
        /// Creates the Hidden Text Field that contains the encoded PreserveProperty
        /// state for all the preserved properties and generates it into the page
        /// as a hidden form variable ( Name of control prefixed with __)
        /// </summary>
        /// <exception>
        /// Fires an ApplicationException if the property cannot be persisted.
        /// </exception>
        public void RenderOutput()
        {
            StringBuilder sb = new StringBuilder();

            foreach (PreservedProperty Property in this.PreservedProperties)
            {
                Control Ctl = Property.ControlInstance;
                if (Ctl == null)
                {
                    Ctl = this.Page.FindControl(Property.ControlId);
                    if (Ctl == null)
                        continue;
                }

                // *** Try to retrieve the property
                object Value = null;
                try
                {
                    Value = Ctl.GetType().InvokeMember(Property.Property,
                                                                 BindingFlags.GetField | BindingFlags.GetProperty |
                                                                 BindingFlags.Instance |
                                                                 BindingFlags.Public | BindingFlags.NonPublic |
                                                                 BindingFlags.IgnoreCase, null, Ctl, null);
                }
                catch
                {
                    throw new ApplicationException("PreserveProperty() couldn't read property " + Property.ControlId + " " + Property.Property);
                }

#if LosFormatter
                string Return = this.LosSerializeObject(Value);
#else
                Type ValueType = Value.GetType();

                // *** Helper manages the conversion to string
                // *** Simple type or ISerializable on - otherwise ApplicationException is thrown
                string Return = this.TypedValueToString(Value, CultureInfo.InvariantCulture);

                // *** If we made it here and we're null the value is not set
                if (Return == null)
                    continue;
#endif
                sb.Append("`|" + Property.ControlId + "~|" + Property.Property + "~|" + Return + "|`");
            }

            string FullOutput = sb.ToString();

            if (this.StoreInSession)
            {
                HttpContext.Current.Session["__PreserveProperty"] = FullOutput;
                return;
            }

            // *** Store in Hidden Form Variable
            if (this.Base64Encode)
                FullOutput = Convert.ToBase64String(Encoding.UTF8.GetBytes(FullOutput));

            this.Page.ClientScript.RegisterHiddenField("__" + this.UniqueID, FullOutput);
        }


        /// <summary>
        /// Converts a type to string if possible. This method supports an optional culture generically on any value.
        /// It calls the ToString() method on common types and uses a type converter on all other objects
        /// if available
        /// </summary>
        /// <param name="RawValue">The Value or Object to convert to a string</param>
        /// <param name="Culture">Culture for numeric and DateTime values</param>
        /// <returns>string</returns>
        public string TypedValueToString(object RawValue, CultureInfo Culture)
        {
            Type ValueType = RawValue.GetType();
            string Return = null;

            if (ValueType == typeof(string))
                Return = RawValue.ToString();
            else if (ValueType == typeof(int) || ValueType == typeof(decimal) ||
                ValueType == typeof(double) || ValueType == typeof(float))
                Return = string.Format(Culture.NumberFormat, "{0}", RawValue);
            else if (ValueType == typeof(DateTime))
                Return = string.Format(Culture.DateTimeFormat, "{0}", RawValue);
            else if (ValueType == typeof(bool))
                Return = RawValue.ToString();
            else if (ValueType == typeof(byte))
                Return = RawValue.ToString();
            else if (ValueType.IsEnum)
                Return = RawValue.ToString();
            else
            {
                Return = this.LosSerializeObject(RawValue);

                //if (ValueType.IsSerializable)
                //{
                //    byte[] BufferResult = null;
                //    if (!wwUtils.SerializeObject(RawValue, out BufferResult))
                //        return "";

                //    Return = Encoding.ASCII.GetString(BufferResult);
                //    //Return = Convert.ToBase64String(BufferResult);
                //}
                //else
                //{
                //    throw new ApplicationException("Can't preserve property of type " + ValueType.Name);

                //    //TypeConverter converter = TypeDescriptor.GetConverter(ValueType);
                //    //if (converter != null && converter.CanConvertTo(typeof(string)))
                //    //    Return = converter.ConvertToString(null, Culture, RawValue);
                //    //else
                //    //    return null;
                //}
            }

            return Return;
        }

        /// <summary>
        /// Turns a string into a typed value. Useful for auto-conversion routines
        /// like form variable or XML parsers.
        /// <seealso>Class wwUtils</seealso>
        /// </summary>
        /// <param name="SourceString">
        /// The string to convert from
        /// </param>
        /// <param name="TargetType">
        /// The type to convert to
        /// </param>
        /// <param name="Culture">
        /// Culture used for numeric and datetime values.
        /// </param>
        /// <returns>object. Throws exception if it cannot be converted.</returns>
        public object StringToTypedValue(string SourceString, Type TargetType, CultureInfo Culture)
        {
            object Result = null;

            if (TargetType == typeof(string))
                Result = SourceString;
            else if (TargetType == typeof(int))
                Result = int.Parse(SourceString, System.Globalization.NumberStyles.Integer, Culture.NumberFormat);
            else if (TargetType == typeof(byte))
                Result = Convert.ToByte(SourceString);
            else if (TargetType == typeof(decimal))
                Result = Decimal.Parse(SourceString, System.Globalization.NumberStyles.Any, Culture.NumberFormat);
            else if (TargetType == typeof(double))
                Result = Double.Parse(SourceString, System.Globalization.NumberStyles.Any, Culture.NumberFormat);
            else if (TargetType == typeof(bool))
            {
                if (SourceString.ToLower() == "true" || SourceString.ToLower() == "on" || SourceString == "1")
                    Result = true;
                else
                    Result = false;
            }
            else if (TargetType == typeof(DateTime))
                Result = Convert.ToDateTime(SourceString, Culture.DateTimeFormat);
            else if (TargetType.IsEnum)
                Result = Enum.Parse(TargetType, SourceString);
            else
            {
                Result = this.LosDeserializeObject(SourceString);
                
                //if (TargetType.IsSerializable)
                //{
                //    if (SourceString == "" || SourceString == "")
                //        return null;

                //    byte[] BufferResult = Encoding.ASCII.GetBytes(SourceString);
                //    //byte[] BufferResult = Convert.FromBase64String(SourceString);
                //    Result = wwUtils.DeSerializeObject(BufferResult, TargetType);
                //}
                //else
                //{

                //    System.ComponentModel.TypeConverter converter = System.ComponentModel.TypeDescriptor.GetConverter(TargetType);
                //    if (converter != null && converter.CanConvertTo(typeof(string)))
                //        try
                //        {
                //            return converter.ConvertFromString(null, Culture, SourceString);

                //        }
                //        catch
                //        { ;   }

                //    throw (new ApplicationException("Type Conversion failed for " +
                //                                    TargetType.Name + "\r\n" + SourceString));
                //}
            }

            return Result;
        }

        private string LosSerializeObject(object obj)
        {
            LosFormatter output = new LosFormatter();
            StringWriter writer = new StringWriter();
            output.Serialize(writer, obj);
            return writer.ToString();
        }

        private object LosDeserializeObject(string inputString)
        {
            LosFormatter input = new LosFormatter();
            return input.Deserialize(inputString);
        }
    }

}
#endif